// src/auth.rs
use super::AppState;
use actix_web::error::{ErrorInternalServerError, ErrorUnauthorized}; // Specific error types
use actix_web::{
    dev::Payload, http::header::AUTHORIZATION, web, Error as ActixWebError, FromRequest,
    HttpRequest,
};
use chrono::Utc;
use futures::future::{ready, Ready};
use log; // Use the log crate
use rusqlite::params;
use rusqlite::Connection;
use std::sync::{Arc, Mutex}; // Import AppState from the parent module (main.rs likely)

// Represents an authenticated user via token
pub struct Auth {
    pub user_id: String,
    pub role: String,
}

impl FromRequest for Auth {
    // Use actix_web::Error for consistency in error handling within Actix
    type Error = ActixWebError;
    // Use Ready from futures 0.3
    type Future = Ready<Result<Self, Self::Error>>;

    fn from_request(req: &HttpRequest, _: &mut Payload) -> Self::Future {
        // Extract database connection pool from application data
        // Extract the *whole* AppState first
        let app_state_result = req.app_data::<web::Data<AppState>>();

        // Get the Arc<Mutex<Connection>> from AppState
        let db_arc_mutex = match app_state_result {
            // Access the 'db' field within the AppState
            Some(data) => data.db.clone(), // Clone the Arc, not the Mutex or Connection
            None => {
                log::error!("Database connection missing in application data configuration.");
                return ready(Err(ErrorInternalServerError(
                    "Internal server error (app configuration)",
                )));
            }
        };

        // Extract Authorization header
        let auth_header = req.headers().get(AUTHORIZATION);

        if let Some(auth_header_value) = auth_header {
            // Convert header value to string
            if let Ok(auth_str) = auth_header_value.to_str() {
                // Check if it starts with "Bearer "
                if auth_str.starts_with("Bearer ") {
                    // Extract the token part
                    let token = &auth_str[7..];

                    // Lock the mutex to get access to the connection
                    // Handle potential mutex poisoning explicitly
                    let conn_guard = match db_arc_mutex.lock() {
                        Ok(guard) => guard,
                        Err(poisoned) => {
                            log::error!("Database mutex poisoned: {}", poisoned);
                            // Return internal server error if mutex is poisoned
                            return ready(Err(ErrorInternalServerError(
                                "Internal server error (database lock)",
                            )));
                        }
                    };

                    // Get user_id and role from token
                    let user_result = conn_guard
                        .query_row(
                            "SELECT u.id, u.role FROM users u WHERE u.token = ?1 AND u.token_expires_at > ?2",
                            params![token, Utc::now().to_rfc3339()],
                            |row| Ok((row.get::<_, String>(0)?, row.get::<_, String>(1)?)),
                        )
                        .optional();

                    match user_result {
                        Ok(Some((user_id, role))) => {
                            log::debug!(
                                "Token validated successfully for user_id: {} with role: {}",
                                user_id,
                                role
                            );
                            ready(Ok(Auth { user_id, role }))
                        }
                        Ok(None) => {
                            log::warn!("Invalid or expired token received");
                            ready(Err(ErrorUnauthorized("Invalid or expired token")))
                        }
                        Err(e) => {
                            log::error!("Database error during token validation: {:?}", e);
                            ready(Err(ErrorUnauthorized("Token validation failed")))
                        }
                    }
                } else {
                    // Header present but not "Bearer " format
                    log::warn!("Invalid Authorization header format (not Bearer)");
                    ready(Err(ErrorUnauthorized("Invalid token format")))
                }
            } else {
                // Header value contains invalid characters
                log::warn!("Authorization header contains invalid characters");
                ready(Err(ErrorUnauthorized("Invalid token value")))
            }
        } else {
            // Authorization header is missing
            log::warn!("Missing Authorization header");
            ready(Err(ErrorUnauthorized("Missing authorization token")))
        }
    }
}

// Helper function to check if a user has admin role
pub fn require_admin(auth: &Auth) -> Result<(), ActixWebError> {
    if auth.role != "admin" {
        return Err(ErrorUnauthorized("Admin access required"));
    }
    Ok(())
}
