// src/db.rs
use anyhow::{anyhow, Context, Result as AnyhowResult};
use bcrypt::{hash, verify, DEFAULT_COST};
use chrono::{Duration as ChronoDuration, Utc}; // Use Utc for timestamps
use log; // Use the log crate
use rusqlite::{params, Connection, OptionalExtension};
use std::env;
use uuid::Uuid;

use crate::models;

// Configurable token lifetime (e.g., from environment variable or default)
const TOKEN_LIFETIME_HOURS: i64 = 24; // Default to 24 hours

// Initialize the database connection and create tables if they don't exist
pub fn init_db(database_url: &str) -> AnyhowResult<Connection> {
    log::info!("Attempting to open or create database at: {}", database_url);
    let conn = Connection::open(database_url)
        .context(format!("Failed to open the database at {}", database_url))?;

    log::debug!("Creating 'users' table if not exists...");
    conn.execute(
        "CREATE TABLE IF NOT EXISTS users (
            id TEXT PRIMARY KEY,
            username TEXT NOT NULL UNIQUE,
            password TEXT NOT NULL,             -- Stores bcrypt hashed password
            role TEXT NOT NULL DEFAULT 'user',  -- 'admin' or 'user'
            token TEXT UNIQUE,                  -- Stores the current session token (UUID)
            token_expires_at DATETIME,          -- Timestamp when the token expires
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP
        )",
        [],
    )
    .context("Failed to create 'users' table")?;

    log::debug!("Creating 'forms' table if not exists...");
    conn.execute(
        "CREATE TABLE IF NOT EXISTS forms (
            id TEXT PRIMARY KEY,
            name TEXT NOT NULL,
            fields TEXT NOT NULL, -- Stores JSON definition of form fields
            owner_id TEXT NOT NULL, -- Reference to the user who created the form
            notify_email TEXT,    -- Optional email address for notifications
            notify_ntfy_topic TEXT, -- Optional ntfy topic for notifications
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (owner_id) REFERENCES users (id) ON DELETE CASCADE
        )",
        [],
    )
    .context("Failed to create 'forms' table")?;

    // Add notify_email column if it doesn't exist (for backward compatibility)
    match conn.execute("ALTER TABLE forms ADD COLUMN notify_email TEXT", []) {
        Ok(_) => log::info!("Added notify_email column to forms table"),
        Err(e) => {
            if !e.to_string().contains("duplicate column name") {
                return Err(anyhow!("Failed to add notify_email column: {}", e));
            }
            // If it already exists, that's fine
        }
    }

    // Add notify_ntfy_topic column if it doesn't exist (for backward compatibility)
    match conn.execute("ALTER TABLE forms ADD COLUMN notify_ntfy_topic TEXT", []) {
        Ok(_) => log::info!("Added notify_ntfy_topic column to forms table"),
        Err(e) => {
            if !e.to_string().contains("duplicate column name") {
                return Err(anyhow!("Failed to add notify_ntfy_topic column: {}", e));
            }
            // If it already exists, that's fine
        }
    }

    log::debug!("Creating 'submissions' table if not exists...");
    conn.execute(
        "CREATE TABLE IF NOT EXISTS submissions (
            id TEXT PRIMARY KEY,
            form_id TEXT NOT NULL,
            data TEXT NOT NULL,    -- Stores JSON submission data
            created_at DATETIME DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (form_id) REFERENCES forms (id) ON DELETE CASCADE
        )",
        [],
    )
    .context("Failed to create 'submissions' table")?;

    // Setup the initial admin user if it doesn't exist, using environment variables
    setup_initial_admin(&conn).context("Failed to setup initial admin user")?;

    log::info!("Database initialization complete.");
    Ok(conn)
}

// Sets up the initial admin user from *required* environment variables if it doesn't exist
fn setup_initial_admin(conn: &Connection) -> AnyhowResult<()> {
    // CRITICAL SECURITY CHANGE: Remove default credentials. Require env vars.
    let initial_admin_username = env::var("INITIAL_ADMIN_USERNAME")
        .map_err(|_| anyhow!("FATAL: INITIAL_ADMIN_USERNAME environment variable is not set."))?;
    let initial_admin_password = env::var("INITIAL_ADMIN_PASSWORD")
        .map_err(|_| anyhow!("FATAL: INITIAL_ADMIN_PASSWORD environment variable is not set."))?;

    if initial_admin_username.is_empty() || initial_admin_password.is_empty() {
        return Err(anyhow!(
            "FATAL: INITIAL_ADMIN_USERNAME and INITIAL_ADMIN_PASSWORD must not be empty."
        ));
    }

    // Check password complexity? (Optional enhancement)

    add_user_if_not_exists(
        conn,
        &initial_admin_username,
        &initial_admin_password,
        Some("admin"),
    )
    .context("Failed during initial admin user setup")?;
    Ok(())
}

// Adds a user with a hashed password if the username doesn't exist
pub fn add_user_if_not_exists(
    conn: &Connection,
    username: &str,
    password: &str,
    role: Option<&str>, // Optional role parameter
) -> AnyhowResult<bool> {
    // Check if user already exists
    let user_exists: bool = conn
        .query_row(
            "SELECT EXISTS(SELECT 1 FROM users WHERE username = ?1)",
            params![username],
            |row| row.get::<_, i32>(0),
        )
        .context(format!("Failed to check existence of user '{}'", username))?
        == 1;

    if user_exists {
        log::debug!("User '{}' already exists, skipping creation.", username);
        return Ok(false); // User already exists, nothing added
    }

    // Generate a UUID for the new user
    let user_id = Uuid::new_v4().to_string();

    // Hash the password using bcrypt
    // Ensure the cost factor is appropriate for your security needs and hardware.
    // Higher cost means slower hashing and verification, but better resistance to brute-force.
    log::debug!(
        "Hashing password for user '{}' with cost {}",
        username,
        DEFAULT_COST
    );
    let hashed_password = hash(password, DEFAULT_COST).context("Failed to hash password")?;

    // Use provided role or default to "user"
    let role = role.unwrap_or("user");

    // Insert the new user
    log::info!(
        "Creating new user '{}' with ID: {} and role: {}",
        username,
        user_id,
        role
    );
    conn.execute(
        "INSERT INTO users (id, username, password, role) VALUES (?1, ?2, ?3, ?4)",
        params![user_id, username, hashed_password, role],
    )
    .context(format!("Failed to insert user '{}'", username))?;

    Ok(true) // User was added
}

// Validate a session token and return the associated user ID if valid and not expired
pub fn validate_token(conn: &Connection, token: &str) -> AnyhowResult<Option<String>> {
    log::debug!("Validating received token (existence and expiration)...");
    let mut stmt = conn.prepare(
        // Select user ID only if token matches AND it hasn't expired
        "SELECT id FROM users WHERE token = ?1 AND token_expires_at IS NOT NULL AND token_expires_at > ?2"
    ).context("Failed to prepare query for validating token")?;

    let now_ts = Utc::now().to_rfc3339(); // Use ISO 8601 / RFC 3339 format compatible with SQLite DATETIME

    let user_id_option: Option<String> = stmt
        .query_row(params![token, now_ts], |row| row.get(0))
        .optional() // Makes it return Option instead of erroring on no rows
        .context("Failed to execute query for validating token")?;

    if user_id_option.is_some() {
        log::debug!("Token validation successful.");
    } else {
        // This covers token not found OR token expired
        log::debug!("Token validation failed (token not found or expired).");
    }

    Ok(user_id_option)
}

// Invalidate a user's token (e.g., on logout) by setting it to NULL and clearing expiration
pub fn invalidate_token(conn: &Connection, user_id: &str) -> AnyhowResult<()> {
    log::debug!("Invalidating token for user_id {}", user_id);
    conn.execute(
        "UPDATE users SET token = NULL, token_expires_at = NULL WHERE id = ?1",
        params![user_id],
    )
    .context(format!(
        "Failed to invalidate token for user_id {}",
        user_id
    ))?;
    Ok(())
}

// Authenticate a user by username and password, returning user ID and hash if successful
pub fn authenticate_user(
    conn: &Connection,
    username: &str,
    password: &str,
) -> AnyhowResult<Option<models::UserAuthData>> {
    log::debug!("Attempting to authenticate user: {}", username);
    let mut stmt = conn
        .prepare("SELECT id, password FROM users WHERE username = ?1")
        .context("Failed to prepare query for authenticating user")?;

    let result = stmt
        .query_row(params![username], |row| {
            Ok(models::UserAuthData {
                id: row.get(0)?,
                hashed_password: row.get(1)?,
            })
        })
        .optional()
        .context(format!(
            "Failed to execute query to fetch auth data for user '{}'",
            username
        ))?;

    match result {
        Some(user_data) => {
            // Verify the provided password against the stored hash
            let is_valid = verify(password, &user_data.hashed_password)
                .context("Failed to verify password hash")?;

            if is_valid {
                log::info!("Authentication successful for user: {}", username);
                Ok(Some(user_data)) // Return user ID and hash
            } else {
                log::warn!(
                    "Authentication failed for user '{}' (invalid password)",
                    username
                );
                Ok(None) // Invalid password
            }
        }
        None => {
            log::warn!(
                "Authentication failed for user '{}' (user not found)",
                username
            );
            Ok(None) // User not found
        }
    }
}

// Generate and save a new session token (with expiration) for a user
pub fn generate_and_set_token_for_user(conn: &Connection, user_id: &str) -> AnyhowResult<String> {
    let new_token = Uuid::new_v4().to_string();
    // Calculate expiration time
    let expires_at = Utc::now() + ChronoDuration::hours(TOKEN_LIFETIME_HOURS);
    let expires_at_ts = expires_at.to_rfc3339(); // Store as string

    log::debug!(
        "Generating new token for user_id {} expiring at {}",
        user_id,
        expires_at_ts
    );

    conn.execute(
        "UPDATE users SET token = ?1, token_expires_at = ?2 WHERE id = ?3",
        params![new_token, expires_at_ts, user_id],
    )
    .context(format!("Failed to update token for user_id {}", user_id))?;

    Ok(new_token)
}

// Fetch a specific form definition by its ID
pub fn get_form_definition(conn: &Connection, form_id: &str) -> AnyhowResult<Option<models::Form>> {
    let mut stmt = conn
        .prepare("SELECT id, name, fields, owner_id, notify_email, notify_ntfy_topic, created_at FROM forms WHERE id = ?1")
        .context("Failed to prepare query for fetching form")?;

    let result = stmt
        .query_row(params![form_id], |row| {
            let id: String = row.get(0)?;
            let name: String = row.get(1)?;
            let fields_str: String = row.get(2)?;
            let owner_id: String = row.get(3)?;
            let notify_email: Option<String> = row.get(4)?;
            let notify_ntfy_topic: Option<String> = row.get(5)?;
            let created_at: chrono::DateTime<chrono::Utc> = row.get(6)?;

            // Parse the fields JSON string
            let fields = serde_json::from_str(&fields_str).map_err(|e| {
                rusqlite::Error::FromSqlConversionFailure(
                    2, // Index of 'fields' column
                    rusqlite::types::Type::Text,
                    Box::new(e),
                )
            })?;

            Ok(models::Form {
                id: Some(id),
                name,
                fields,
                owner_id,
                notify_email,
                notify_ntfy_topic,
                created_at,
            })
        })
        .optional()
        .context(format!("Failed to fetch form with ID: {}", form_id))?;

    Ok(result)
}

// Add a function to save a form
impl models::Form {
    pub fn save(&self, conn: &Connection) -> AnyhowResult<()> {
        let id = self
            .id
            .clone()
            .unwrap_or_else(|| Uuid::new_v4().to_string());
        let fields_json = serde_json::to_string(&self.fields)?;

        conn.execute(
            "INSERT INTO forms (id, name, fields, owner_id, notify_email, notify_ntfy_topic, created_at)
             VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7)
             ON CONFLICT(id) DO UPDATE SET
                name = excluded.name,
                fields = excluded.fields,
                owner_id = excluded.owner_id,
                notify_email = excluded.notify_email,
                notify_ntfy_topic = excluded.notify_ntfy_topic",
            params![
                id,
                self.name,
                fields_json,
                self.owner_id,
                self.notify_email,
                self.notify_ntfy_topic,
                self.created_at
            ],
        )?;

        Ok(())
    }

    pub fn get_by_id(conn: &Connection, id: &str) -> AnyhowResult<Self> {
        get_form_definition(conn, id)?.ok_or_else(|| anyhow!("Form not found: {}", id))
    }
}

// Add a function to save a submission
impl models::Submission {
    pub fn save(&self, conn: &Connection) -> AnyhowResult<()> {
        let data_json = serde_json::to_string(&self.data)?;

        conn.execute(
            "INSERT INTO submissions (id, form_id, data, created_at) 
             VALUES (?1, ?2, ?3, ?4)",
            params![self.id, self.form_id, data_json, self.created_at],
        )?;

        Ok(())
    }
}

// Get user by ID
pub fn get_user_by_id(conn: &Connection, user_id: &str) -> AnyhowResult<Option<models::User>> {
    let mut stmt =
        conn.prepare("SELECT id, username, role, created_at FROM users WHERE id = ?1")?;

    let result = stmt
        .query_row(params![user_id], |row| {
            Ok(models::User {
                id: row.get(0)?,
                username: row.get(1)?,
                password: None, // Never return password
                role: row.get(2)?,
                created_at: row.get(3)?,
            })
        })
        .optional()?;

    Ok(result)
}

// Get user by username
pub fn get_user_by_username(
    conn: &Connection,
    username: &str,
) -> AnyhowResult<Option<models::User>> {
    let mut stmt =
        conn.prepare("SELECT id, username, role, created_at FROM users WHERE username = ?1")?;

    let result = stmt
        .query_row(params![username], |row| {
            Ok(models::User {
                id: row.get(0)?,
                username: row.get(1)?,
                password: None, // Never return password
                role: row.get(2)?,
                created_at: row.get(3)?,
            })
        })
        .optional()?;

    Ok(result)
}

// List all users (for admin use)
pub fn list_users(conn: &Connection) -> AnyhowResult<Vec<models::User>> {
    let mut stmt = conn.prepare("SELECT id, username, role, created_at FROM users")?;

    let users_iter = stmt.query_map([], |row| {
        Ok(models::User {
            id: row.get(0)?,
            username: row.get(1)?,
            password: None, // Never return password
            role: row.get(2)?,
            created_at: row.get(3)?,
        })
    })?;

    let mut users = Vec::new();
    for user_result in users_iter {
        users.push(user_result?);
    }

    Ok(users)
}

// Update user
pub fn update_user(
    conn: &Connection,
    user_id: &str,
    update: &models::UserUpdate,
) -> AnyhowResult<()> {
    if let Some(username) = &update.username {
        conn.execute(
            "UPDATE users SET username = ?1 WHERE id = ?2",
            params![username, user_id],
        )?;
    }

    if let Some(password) = &update.password {
        let hashed_password = hash(password, DEFAULT_COST)?;
        conn.execute(
            "UPDATE users SET password = ?1 WHERE id = ?2",
            params![hashed_password, user_id],
        )?;
    }

    Ok(())
}

// Delete user
pub fn delete_user(conn: &Connection, user_id: &str) -> AnyhowResult<bool> {
    let rows_affected = conn.execute("DELETE FROM users WHERE id = ?1", params![user_id])?;

    Ok(rows_affected > 0)
}
