use crate::auth::Auth;
use crate::models::{
    Form, LoginCredentials, LoginResponse, Submission, User, UserRegistration, UserUpdate,
};
use crate::AppState;
use actix_web::{web, Error as ActixWebError, HttpResponse, Responder, Result as ActixResult};
use chrono; // Only import the module since we use it qualified
use log;
use regex::Regex; // For pattern validation
use rusqlite::{params, Connection};
use serde_json::{json, Map, Value as JsonValue}; // Alias for clarity
use std::collections::HashMap;
use std::sync::{Arc, Mutex};
use uuid::Uuid;

// Added imports for CAPTCHA verification
use actix_web::HttpRequest;
use reqwest;
use serde::Deserialize;

// Added for throttling
use std::time::{Duration, Instant};

// --- Struct for CAPTCHA Verification Response ---
#[derive(Deserialize, Debug)]
struct CaptchaVerificationResponse {
    success: bool,
    // Providers might include other fields like challenge_ts, hostname, error-codes
    #[serde(rename = "error-codes")]
    error_codes: Option<Vec<String>>,
}

// --- Helper Function for Validation ---

/// Validates submission data against the form field definitions with enhanced checks.
///
/// Expected field definition properties:
/// - `name`: string (required)
/// - `type`: string (e.g., "string", "number", "boolean", "email", "url", "object", "array") (required)
/// - `required`: boolean (optional, default: false)
/// - `maxLength`: number (for "string" type)
/// - `minLength`: number (for "string" type)
/// - `min`: number (for "number" type)
/// - `max`: number (for "number" type)
/// - `pattern`: string (regex for "string", "email", "url" types)
///
/// Returns `Ok(())` if valid, or `Err(JsonValue)` containing validation errors.
fn validate_submission_against_definition(
    submission_data: &JsonValue,
    form_definition_fields: &JsonValue,
) -> Result<(), JsonValue> {
    let mut errors: HashMap<String, String> = HashMap::new();

    // Ensure 'fields' in the definition is a JSON array
    let field_definitions = match form_definition_fields.as_array() {
        Some(defs) => defs,
        None => {
            log::error!(
                "Form definition 'fields' is not a JSON array. Def: {:?}",
                form_definition_fields
            );
            errors.insert(
                "_internal".to_string(),
                "Invalid form definition format (not an array)".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Ensure the submission data is a JSON object
    let data_map = match submission_data.as_object() {
        Some(map) => map,
        None => {
            errors.insert(
                "_submission".to_string(),
                "Submission data must be a JSON object".to_string(),
            );
            return Err(json!({ "validation_errors": errors }));
        }
    };

    // Build a map of valid field names to their definitions from the definition for quick lookup
    let defined_field_names: HashMap<String, &Map<String, JsonValue>> = field_definitions
        .iter()
        .filter_map(|val| val.as_object())
        .filter_map(|def| {
            def.get("name")
                .and_then(JsonValue::as_str)
                .map(|name| (name.to_string(), def))
        })
        .collect();

    // 1. Check for submitted fields that are NOT in the definition
    for submitted_key in data_map.keys() {
        if !defined_field_names.contains_key(submitted_key) {
            errors.insert(
                submitted_key.clone(),
                "Unexpected field submitted".to_string(),
            );
        }
    }
    // Exit early if unexpected fields were found
    if !errors.is_empty() {
        log::warn!("Submission validation failed: Unexpected fields submitted.");
        return Err(json!({ "validation_errors": errors }));
    }

    // 2. Iterate through each field definition and validate corresponding submitted data
    for (field_name, field_def) in &defined_field_names {
        // Extract properties using helper functions for clarity
        let field_type = field_def
            .get("type")
            .and_then(JsonValue::as_str)
            .unwrap_or("string"); // Default to "string" if type is missing or not a string
        let is_required = field_def
            .get("required")
            .and_then(JsonValue::as_bool)
            .unwrap_or(false); // Default to false if required is missing or not a boolean
        let min_length = field_def.get("minLength").and_then(JsonValue::as_u64);
        let max_length = field_def.get("maxLength").and_then(JsonValue::as_u64);
        let min_value = field_def.get("min").and_then(JsonValue::as_f64); // Use f64 for flexibility
        let max_value = field_def.get("max").and_then(JsonValue::as_f64);
        let pattern = field_def.get("pattern").and_then(JsonValue::as_str);

        match data_map.get(field_name) {
            Some(submitted_value) if !submitted_value.is_null() => {
                // Field is present and not null, perform type and constraint checks
                let mut type_error = None;
                let mut constraint_errors = vec![];

                match field_type {
                    "string" | "email" | "url" => {
                        if let Some(s) = submitted_value.as_str() {
                            if let Some(min) = min_length {
                                if (s.chars().count() as u64) < min {
                                    // Use chars().count() for UTF-8 correctness
                                    constraint_errors
                                        .push(format!("Must be at least {} characters long", min));
                                }
                            }
                            if let Some(max) = max_length {
                                if (s.chars().count() as u64) > max {
                                    constraint_errors.push(format!(
                                        "Must be no more than {} characters long",
                                        max
                                    ));
                                }
                            }
                            if let Some(pat) = pattern {
                                // Consider caching compiled Regex if performance is critical
                                // and patterns are reused frequently across requests.
                                match Regex::new(pat) {
                                    Ok(re) => {
                                        if !re.is_match(s) {
                                           constraint_errors.push(format!("Does not match required pattern"));
                                        }
                                    }
                                    Err(e) => log::warn!("Invalid regex pattern '{}' in form definition for field '{}': {}", pat, field_name, e), // Log regex compilation error
                                }
                            }
                            // Specific checks for email/url
                            if field_type == "email" {
                                // Basic email regex (adjust for stricter needs or use a validation crate)
                                // This regex is very basic and allows many technically invalid addresses.
                                // Consider crates like `validator` for more robust validation.
                                let email_regex =
                                    Regex::new(r"^[^\s@]+@[^\s@]+\.[^\s@]+$").unwrap(); // Safe unwrap for known valid regex
                                if !email_regex.is_match(s) {
                                    constraint_errors
                                        .push("Must be a valid email address".to_string());
                                }
                            }
                            if field_type == "url" {
                                // Basic URL check (consider `url` crate for robustness)
                                if url::Url::parse(s).is_err() {
                                    constraint_errors.push("Must be a valid URL".to_string());
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a string for '{}'", field_name));
                        }
                    }
                    "number" => {
                        // Use as_f64 for flexibility (handles integers and floats)
                        if let Some(num) = submitted_value.as_f64() {
                            if let Some(min) = min_value {
                                if num < min {
                                    constraint_errors.push(format!("Must be at least {}", min));
                                }
                            }
                            if let Some(max) = max_value {
                                if num > max {
                                    constraint_errors.push(format!("Must be no more than {}", max));
                                }
                            }
                        } else {
                            type_error = Some(format!("Expected a number for '{}'", field_name));
                        }
                    }
                    "boolean" => {
                        if !submitted_value.is_boolean() {
                            type_error = Some(format!(
                                "Expected a boolean (true/false) for '{}'",
                                field_name
                            ));
                        }
                    }
                    "object" => {
                        if !submitted_value.is_object() {
                            type_error =
                                Some(format!("Expected a JSON object for '{}'", field_name));
                        }
                        // TODO: Could add deeper validation for object structure here if needed based on definition
                    }
                    "array" => {
                        if !submitted_value.is_array() {
                            type_error =
                                Some(format!("Expected a JSON array for '{}'", field_name));
                        }
                        // TODO: Could add validation for array elements here if needed based on definition
                    }
                    _ => {
                        // Log unsupported types during development/debugging if necessary
                        log::trace!(
                            "Unsupported field type '{}' encountered during validation for field '{}'. Treating as valid.",
                            field_type,
                            field_name
                        );
                        // Assume valid if type is not specifically handled or unknown
                    }
                }

                // Record errors found for this field
                if let Some(err) = type_error {
                    errors.insert(field_name.clone(), err);
                } else if !constraint_errors.is_empty() {
                    // Combine multiple constraint errors if necessary
                    errors.insert(field_name.clone(), constraint_errors.join("; "));
                }
            } // End check for present and non-null value
            Some(_) => {
                // Value is present but explicitly null (e.g., "fieldName": null)
                if is_required {
                    errors.insert(
                        field_name.clone(),
                        "This field is required and cannot be null".to_string(),
                    );
                }
                // Otherwise, null is considered a valid (empty) value for non-required fields
            }
            None => {
                // Field is missing entirely from the submission object
                if is_required {
                    errors.insert(field_name.clone(), "This field is required".to_string());
                }
                // Missing is valid for non-required fields
            }
        } // End match data_map.get(field_name)
    } // End loop through field definitions

    // Check if any errors were collected
    if errors.is_empty() {
        Ok(()) // Validation passed
    } else {
        log::info!(
            "Submission validation failed with {} error(s).", // Log only the count for brevity
            errors.len()
        );
        // Return a JSON object containing the specific validation errors
        Err(json!({ "validation_errors": errors }))
    }
}

// Helper function to convert anyhow::Error to actix_web::Error
fn anyhow_to_actix_error(e: anyhow::Error) -> ActixWebError {
    actix_web::error::ErrorInternalServerError(e.to_string())
}

// --- Public Handlers ---

// POST /login
pub async fn login(
    app_state: web::Data<AppState>, // Expect AppState like other handlers
    creds: web::Json<LoginCredentials>,
) -> ActixResult<impl Responder> {
    // Clone the Arc<Mutex<Connection>> from AppState
    let db_conn_arc = app_state.db.clone();
    let username = creds.username.clone();
    let password = creds.password.clone();

    // Wrap the blocking database operations in web::block
    let auth_result = web::block(move || {
        // Use the cloned Arc here
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during login lock"))?;
        crate::db::authenticate_user(&conn, &username, &password)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error during authentication: {:?}", e);
        actix_web::error::ErrorInternalServerError("Authentication process failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    match auth_result {
        Some(user_data) => {
            // Clone Arc again for token generation, using the AppState db field
            let db_conn_token_arc = app_state.db.clone();
            let user_id = user_data.id.clone();

            // Generate and store a new token within web::block
            let token = web::block(move || {
                // Use the cloned Arc here
                let conn = db_conn_token_arc
                    .lock()
                    .map_err(|_| anyhow::anyhow!("Database mutex poisoned during token lock"))?;
                crate::db::generate_and_set_token_for_user(&conn, &user_id)
            })
            .await
            .map_err(|e| {
                log::error!("web::block error during token generation: {:?}", e);
                actix_web::error::ErrorInternalServerError(
                    "Failed to complete login (token generation blocking error)",
                )
            })?
            .map_err(anyhow_to_actix_error)?;

            log::info!("Login successful for user_id: {}", user_data.id);
            Ok(HttpResponse::Ok().json(LoginResponse { token }))
        }
        None => {
            log::warn!("Login failed for username: {}", creds.username);
            // Return 401 Unauthorized for failed login attempts
            Err(actix_web::error::ErrorUnauthorized(
                "Invalid username or password",
            ))
        }
    }
}

// POST /logout
pub async fn logout(
    app_state: web::Data<AppState>, // Expect AppState
    auth: Auth,                     // Requires authentication (extracts user_id from token)
) -> ActixResult<impl Responder> {
    log::info!("User {} requesting logout", auth.user_id);
    let db_conn_arc = app_state.db.clone(); // Get db from AppState
    let user_id = auth.user_id.clone();

    // Invalidate the token in the database within web::block
    web::block(move || {
        let conn = db_conn_arc // Use the cloned Arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during logout lock"))?;
        crate::db::invalidate_token(&conn, &user_id)
    })
    .await
    .map_err(|e| {
        // Use the original auth.user_id here as user_id moved into the block
        log::error!(
            "web::block error during logout for user {}: {:?}",
            auth.user_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Logout failed (blocking error)")
    })?
    .map_err(anyhow_to_actix_error)?;

    log::info!("User {} logged out successfully", auth.user_id);
    Ok(HttpResponse::Ok().json(json!({ "message": "Logged out successfully" })))
}

// POST /forms/{form_id}/submissions
pub async fn submit_form(
    req: HttpRequest, // Add HttpRequest to access connection info
    app_state: web::Data<AppState>,
    path: web::Path<String>,                  // Extracts form_id from path
    submission_payload: web::Json<JsonValue>, // Expect arbitrary JSON payload
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    // Use .get_ref() to borrow AppState without consuming web::Data
    let app_state_ref = app_state.get_ref();
    let captcha_config = &app_state_ref.captcha_config;

    // --- Per-Form Per-IP Rate Limiting ---
    const RATE_LIMIT_DURATION: Duration = Duration::from_secs(60); // 1 minute window
    const RATE_LIMIT_MAX_ATTEMPTS: u32 = 5; // Max 5 attempts per window

    let client_ip_opt = req
        .connection_info()
        .realip_remote_addr()
        .map(|s| s.to_string());

    if let Some(client_ip) = client_ip_opt {
        let mut attempts_map = app_state_ref.form_submission_attempts.lock().map_err(|e| {
            log::error!("Failed to acquire rate limit lock: {}", e);
            actix_web::error::ErrorInternalServerError("Internal error (rate limit state)")
        })?;

        let now = Instant::now();
        let form_attempts = attempts_map.entry(form_id.clone()).or_default();
        let (last_attempt, count) = form_attempts.entry(client_ip.clone()).or_insert((now, 0));

        if now.duration_since(*last_attempt) > RATE_LIMIT_DURATION {
            // Reset count if window expired
            *last_attempt = now;
            *count = 1;
        } else {
            // Increment count within the window
            *count += 1;
        }

        log::debug!(
            "Rate limit check for form '{}', IP '{}': attempt count = {}, last attempt = {:?}",
            form_id,
            client_ip,
            *count,
            last_attempt
        );

        if *count > RATE_LIMIT_MAX_ATTEMPTS {
            log::warn!(
                "Rate limit exceeded for form '{}', IP '{}'. Count: {}. Blocking request.",
                form_id,
                client_ip,
                *count
            );
            // Consider clearing the entry after a longer block duration if needed
            return Ok(HttpResponse::TooManyRequests().json(json!({
                "error": "rate_limit_exceeded",
                "message": "Too many submission attempts. Please try again later."
            })));
        }
    } else {
        // Cannot rate limit if IP address is unknown
        log::warn!("Could not determine client IP for rate limiting.");
    }
    // --- End Rate Limiting ---

    let payload_value = submission_payload.into_inner(); // Get the owned JsonValue

    // --- CAPTCHA Verification ---
    if captcha_config.enabled {
        let captcha_token = payload_value.get("captcha_token").and_then(|v| v.as_str());

        match captcha_token {
            Some(token) if !token.is_empty() => {
                // Get client IP address
                let client_ip = req
                    .connection_info()
                    .realip_remote_addr()
                    .map(|s| s.to_string());
                // Note: Ensure Actix is configured correctly behind a proxy if needed
                // using .forwarded_for() or similar mechanisms if realip_remote_addr() isn't sufficient.

                log::debug!(
                    "Verifying CAPTCHA token for IP: {:?}",
                    client_ip.as_deref().unwrap_or("Unknown")
                );

                let mut params = HashMap::new();
                params.insert("secret", captcha_config.secret_key.as_str());
                params.insert("response", token);
                if let Some(ip) = client_ip.as_deref() {
                    params.insert("remoteip", ip);
                }

                // Consider creating the client once and storing it in AppState for reuse
                let client = reqwest::Client::new();
                let res = client
                    .post(&captcha_config.verification_url)
                    .form(&params)
                    .send()
                    .await;

                match res {
                    Ok(response) => {
                        if response.status().is_success() {
                            match response.json::<CaptchaVerificationResponse>().await {
                                Ok(verification_response) => {
                                    if verification_response.success {
                                        log::info!("CAPTCHA verification successful.");
                                    } else {
                                        log::warn!(
                                            "CAPTCHA verification failed: {:?}",
                                            verification_response.error_codes
                                        );
                                        return Ok(HttpResponse::BadRequest().json(json!({
                                            "error": "captcha_verification_failed",
                                            "message": "Invalid CAPTCHA token."
                                        })));
                                    }
                                }
                                Err(e) => {
                                    log::error!(
                                        "Failed to parse CAPTCHA verification response: {}",
                                        e
                                    );
                                    return Ok(HttpResponse::InternalServerError().json(json!({
                                        "error": "captcha_provider_error",
                                        "message": "Failed to process CAPTCHA provider response."
                                    })));
                                }
                            }
                        } else {
                            log::error!(
                                "CAPTCHA provider request failed with status: {}",
                                response.status()
                            );
                            return Ok(HttpResponse::InternalServerError().json(json!({
                                "error": "captcha_provider_error",
                                "message": "Could not reach CAPTCHA provider."
                            })));
                        }
                    }
                    Err(e) => {
                        log::error!("Failed to send CAPTCHA verification request: {}", e);
                        return Ok(HttpResponse::InternalServerError().json(json!({
                            "error": "captcha_provider_error",
                            "message": "Failed to send request to CAPTCHA provider."
                        })));
                    }
                }
            }
            _ => {
                log::warn!("CAPTCHA enabled, but no valid token provided in submission.");
                return Ok(HttpResponse::BadRequest().json(json!({ "error": "captcha_token_missing", "message": "CAPTCHA token is required."})));
            }
        }
    }
    // --- End CAPTCHA Verification ---

    // Lock DB connection AFTER CAPTCHA check
    // Use app_state_ref here as well
    let conn = app_state_ref.db.lock().map_err(|e| {
        log::error!("Failed to acquire database lock: {}", e);
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Get form definition
    let form = Form::get_by_id(&conn, &form_id).map_err(anyhow_to_actix_error)?;

    // Validate submission against form definition (using the owned payload_value)
    if let Err(validation_errors) =
        validate_submission_against_definition(&payload_value, &form.fields)
    {
        return Ok(HttpResponse::BadRequest().json(validation_errors));
    }

    // Create submission record
    let submission = Submission {
        id: Uuid::new_v4().to_string(),
        form_id: form_id.clone(),
        data: payload_value, // Store the full validated payload (including captcha_token if sent)
        created_at: chrono::Utc::now(),
    };

    // Save submission to database
    submission.save(&conn).map_err(|e| {
        log::error!("Failed to save submission: {}", e);
        actix_web::error::ErrorInternalServerError("Failed to save submission")
    })?;

    // --- Notification Throttling & Sending ---
    const NOTIFICATION_THROTTLE_DURATION: Duration = Duration::from_secs(60);
    let mut should_send_notification = true; // Assume we should send initially

    // Check if notifications are configured for this form at all
    let notifications_configured = form.notify_email.is_some()
        || form
            .notify_ntfy_topic
            .as_ref()
            .map_or(false, |s| !s.is_empty());

    if notifications_configured {
        let mut last_times = app_state_ref.last_notification_times.lock().map_err(|e| {
            log::error!("Failed to acquire notification throttle lock: {}", e);
            actix_web::error::ErrorInternalServerError("Internal error (notification state)")
        })?;

        let now = Instant::now();
        if let Some(last_time) = last_times.get(&form_id) {
            if now.duration_since(*last_time) < NOTIFICATION_THROTTLE_DURATION {
                log::info!(
                    "Notification throttled for form_id: {}. Last sent {:?} ago.",
                    form_id,
                    now.duration_since(*last_time)
                );
                should_send_notification = false;
            }
        }

        // If not throttled, update the timestamp *before* attempting to send
        if should_send_notification {
            log::debug!("Updating last notification time for form_id: {}", form_id);
            last_times.insert(form_id.clone(), now);
        }
    } else {
        should_send_notification = false; // Don't attempt if not configured
    }

    // Send notifications only if not throttled and configured
    if should_send_notification {
        log::info!("Attempting to send notifications for form_id: {}", form_id);
        // Send Email if configured
        if let Some(notify_email) = &form.notify_email {
            let email_subject = format!("New submission for form: {}", form.name);
            let email_body = format!(
                "A new submission has been received for form '{}'.\n\nSubmission ID: {}\nTimestamp: {}\n\nData:\n{}",
                form.name,
                submission.id,
                submission.created_at,
                serde_json::to_string_pretty(&submission.data).unwrap_or_default()
            );
            // Use a clone of notification_service if it needs to move into async block
            let notification_service_clone = app_state_ref.notification_service.clone();
            let notify_email_clone = notify_email.clone();
            let email_subject_clone = email_subject.clone();
            let email_body_clone = email_body.clone();

            // Spawn email sending as a background task so it doesn't block the response
            tokio::spawn(async move {
                if let Err(e) = notification_service_clone
                    .send_email(&notify_email_clone, &email_subject_clone, &email_body_clone)
                    .await
                {
                    log::warn!(
                        "Failed to send email notification in background task: {}",
                        e
                    );
                }
            });
        }

        // Send ntfy if configured
        if let Some(topic_flag) = &form.notify_ntfy_topic {
            if !topic_flag.is_empty() {
                let ntfy_title = format!("New submission for: {}", form.name);
                let ntfy_message = format!("Form: {}\nSubmission ID: {}", form.name, submission.id);
                // Ntfy send is synchronous in the current implementation, can block
                // Consider spawning if it becomes slow
                if let Err(e) = app_state_ref.notification_service.send_ntfy(
                    &ntfy_title,
                    &ntfy_message,
                    Some(3), // Medium priority
                ) {
                    log::warn!("Failed to send ntfy notification: {}", e);
                    // Don't return error to client, just log
                }
            }
        }
    } // End if should_send_notification
      // --- End Notification Throttling & Sending ---

    Ok(HttpResponse::Created().json(json!({
        "message": "Submission received",
        "submission_id": submission.id
    })))
}

// POST /forms
pub async fn create_form(
    app_state: web::Data<AppState>,
    auth: Auth,
    form_data: web::Json<Form>,
) -> ActixResult<impl Responder> {
    let mut form = form_data.into_inner();
    form.owner_id = auth.user_id.clone(); // Set the owner_id to the authenticated user's ID

    let db_conn_arc = app_state.db.clone();
    web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;
        form.save(&conn)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while creating form: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to create form")
    })?
    .map_err(anyhow_to_actix_error)?;

    Ok(HttpResponse::Created().json(form))
}

// GET /forms
pub async fn get_forms(app_state: web::Data<AppState>, auth: Auth) -> ActixResult<impl Responder> {
    let db_conn_arc = app_state.db.clone();
    let user_id = auth.user_id.clone();
    let is_admin = auth.role == "admin";

    let forms = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;

        let mut stmt = if is_admin {
            // Admins can see all forms
            conn.prepare("SELECT id, name, fields, owner_id, notify_email, notify_ntfy_topic, created_at FROM forms")?
        } else {
            // Regular users can only see their own forms
            conn.prepare("SELECT id, name, fields, owner_id, notify_email, notify_ntfy_topic, created_at FROM forms WHERE owner_id = ?1")?
        };

        let forms_iter = if is_admin {
            stmt.query_map([], |row| {
                let id: String = row.get(0)?;
                let name: String = row.get(1)?;
                let fields_str: String = row.get(2)?;
                let owner_id: String = row.get(3)?;
                let notify_email: Option<String> = row.get(4)?;
                let notify_ntfy_topic: Option<String> = row.get(5)?;
                let created_at: chrono::DateTime<chrono::Utc> = row.get(6)?;

                let fields = serde_json::from_str(&fields_str).map_err(|e| {
                    rusqlite::Error::FromSqlConversionFailure(
                        2,
                        rusqlite::types::Type::Text,
                        Box::new(e),
                    )
                })?;

                Ok(Form {
                    id: Some(id),
                    name,
                    fields,
                    owner_id,
                    notify_email,
                    notify_ntfy_topic,
                    created_at,
                })
            })?
        } else {
            stmt.query_map(params![user_id], |row| {
                let id: String = row.get(0)?;
                let name: String = row.get(1)?;
                let fields_str: String = row.get(2)?;
                let owner_id: String = row.get(3)?;
                let notify_email: Option<String> = row.get(4)?;
                let notify_ntfy_topic: Option<String> = row.get(5)?;
                let created_at: chrono::DateTime<chrono::Utc> = row.get(6)?;

                let fields = serde_json::from_str(&fields_str).map_err(|e| {
                    rusqlite::Error::FromSqlConversionFailure(
                        2,
                        rusqlite::types::Type::Text,
                        Box::new(e),
                    )
                })?;

                Ok(Form {
                    id: Some(id),
                    name,
                    fields,
                    owner_id,
                    notify_email,
                    notify_ntfy_topic,
                    created_at,
                })
            })?
        };

        let mut forms = Vec::new();
        for form_result in forms_iter {
            forms.push(form_result?);
        }

        Ok::<_, anyhow::Error>(forms)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while fetching forms: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to fetch forms")
    })?
    .map_err(anyhow_to_actix_error)?;

    Ok(HttpResponse::Ok().json(forms))
}

// GET /forms/{form_id}/submissions
pub async fn get_submissions(
    app_state: web::Data<AppState>,
    auth: Auth,
    form_id: web::Path<String>,
) -> ActixResult<impl Responder> {
    let db_conn_arc = app_state.db.clone();
    let form_id_str = form_id.into_inner();
    let user_id = auth.user_id.clone();
    let is_admin = auth.role == "admin";

    // First check if the user has access to this form
    let can_access = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;

        if is_admin {
            // Admins can access all forms
            return Ok(true);
        }

        // Check if the form belongs to the user
        let owner_id: Option<String> = conn
            .query_row(
                "SELECT owner_id FROM forms WHERE id = ?1",
                params![form_id_str],
                |row| row.get(0),
            )
            .optional()?;

        match owner_id {
            Some(owner_id) => Ok(owner_id == user_id),
            None => Ok(false), // Form doesn't exist
        }
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while checking form access: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to check form access")
    })?
    .map_err(anyhow_to_actix_error)?;

    if !can_access {
        return Err(actix_web::error::ErrorForbidden("Access denied"));
    }

    // Now fetch the submissions
    let db_conn_arc = app_state.db.clone();
    let form_id_str = form_id.into_inner();

    let submissions = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;

        let mut stmt = conn
            .prepare("SELECT id, form_id, data, created_at FROM submissions WHERE form_id = ?1")?;

        let submissions_iter = stmt.query_map(params![form_id_str], |row| {
            let id: String = row.get(0)?;
            let form_id: String = row.get(1)?;
            let data_str: String = row.get(2)?;
            let created_at: chrono::DateTime<chrono::Utc> = row.get(3)?;

            let data = serde_json::from_str(&data_str).map_err(|e| {
                rusqlite::Error::FromSqlConversionFailure(
                    2,
                    rusqlite::types::Type::Text,
                    Box::new(e),
                )
            })?;

            Ok(Submission {
                id,
                form_id,
                data,
                created_at,
            })
        })?;

        let mut submissions = Vec::new();
        for submission_result in submissions_iter {
            submissions.push(submission_result?);
        }

        Ok::<_, anyhow::Error>(submissions)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while fetching submissions: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to fetch submissions")
    })?
    .map_err(anyhow_to_actix_error)?;

    Ok(HttpResponse::Ok().json(submissions))
}

// --- Notification Settings Handlers ---

// GET /forms/{form_id}/notifications
pub async fn get_notification_settings(
    app_state: web::Data<AppState>,
    auth: Auth, // Requires authentication
    path: web::Path<String>,
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    log::info!(
        "User {} requesting notification settings for form_id: {}",
        auth.user_id,
        form_id
    );

    let conn = app_state.db.lock().map_err(|e| {
        log::error!(
            "Failed to acquire database lock for get_notification_settings: {}",
            e
        );
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Get the form to ensure it exists and retrieve current settings
    let form = Form::get_by_id(&conn, &form_id).map_err(|e| {
        log::warn!(
            "Attempt to get settings for non-existent form {}: {}",
            form_id,
            e
        );
        if e.to_string().contains("not found") {
            actix_web::error::ErrorNotFound("Form not found")
        } else {
            actix_web::error::ErrorInternalServerError("Database error retrieving form")
        }
    })?;

    let settings = crate::models::NotificationSettingsPayload {
        notify_email: form.notify_email,
        notify_ntfy_topic: form.notify_ntfy_topic,
    };

    Ok(HttpResponse::Ok().json(settings))
}

// PUT /forms/{form_id}/notifications
pub async fn update_notification_settings(
    app_state: web::Data<AppState>,
    auth: Auth, // Requires authentication
    path: web::Path<String>,
    payload: web::Json<crate::models::NotificationSettingsPayload>,
) -> ActixResult<impl Responder> {
    let form_id = path.into_inner();
    let new_settings = payload.into_inner();
    log::info!(
        "User {} updating notification settings for form_id: {}. Settings: {:?}",
        auth.user_id,
        form_id,
        new_settings
    );

    let conn = app_state.db.lock().map_err(|e| {
        log::error!(
            "Failed to acquire database lock for update_notification_settings: {}",
            e
        );
        actix_web::error::ErrorInternalServerError("Database error")
    })?;

    // Fetch the existing form to update it
    let mut form = Form::get_by_id(&conn, &form_id).map_err(|e| {
        log::warn!(
            "Attempt to update settings for non-existent form {}: {}",
            form_id,
            e
        );
        if e.to_string().contains("not found") {
            actix_web::error::ErrorNotFound("Form not found")
        } else {
            actix_web::error::ErrorInternalServerError("Database error retrieving form")
        }
    })?;

    // Update the form fields
    form.notify_email = new_settings.notify_email;
    form.notify_ntfy_topic = new_settings.notify_ntfy_topic;

    // Save the updated form
    form.save(&conn).map_err(|e| {
        log::error!(
            "Failed to save updated notification settings for form {}: {}",
            form_id,
            e
        );
        actix_web::error::ErrorInternalServerError("Failed to save notification settings")
    })?;

    log::info!(
        "Successfully updated notification settings for form {}",
        form_id
    );
    Ok(HttpResponse::Ok().json(json!({ "message": "Notification settings updated successfully" })))
}

pub async fn health_check() -> impl Responder {
    HttpResponse::Ok().json(serde_json::json!({
        "status": "ok",
        "version": env!("CARGO_PKG_VERSION"),
        "timestamp": chrono::Utc::now().to_rfc3339()
    }))
}

// POST /register
pub async fn register(
    app_state: web::Data<AppState>,
    registration: web::Json<models::UserRegistration>,
) -> ActixResult<impl Responder> {
    let db_conn_arc = app_state.db.clone();
    let username = registration.username.clone();
    let password = registration.password.clone();

    // Register user in a blocking operation
    let result = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned during registration"))?;

        // Check if username already exists
        if let Some(_) = crate::db::get_user_by_username(&conn, &username)? {
            return Err(anyhow::anyhow!("Username already exists"));
        }

        // Add new user with default role "user"
        crate::db::add_user_if_not_exists(&conn, &username, &password, None)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error during registration: {:?}", e);
        actix_web::error::ErrorInternalServerError("Registration process failed")
    })?
    .map_err(|e| {
        if e.to_string().contains("already exists") {
            actix_web::error::ErrorConflict(e.to_string())
        } else {
            actix_web::error::ErrorInternalServerError(e.to_string())
        }
    })?;

    Ok(HttpResponse::Created().json(json!({
        "message": "User registered successfully"
    })))
}

// GET /users (admin only)
pub async fn list_users(app_state: web::Data<AppState>, auth: Auth) -> ActixResult<impl Responder> {
    // Check admin role
    crate::auth::require_admin(&auth)?;

    let db_conn_arc = app_state.db.clone();

    let users = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;
        crate::db::list_users(&conn)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while listing users: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to list users")
    })?
    .map_err(anyhow_to_actix_error)?;

    Ok(HttpResponse::Ok().json(users))
}

// GET /users/{user_id} (admin or self)
pub async fn get_user(
    app_state: web::Data<AppState>,
    auth: Auth,
    user_id: web::Path<String>,
) -> ActixResult<impl Responder> {
    // Allow if admin or if user is requesting their own data
    if auth.role != "admin" && auth.user_id != user_id.as_str() {
        return Err(actix_web::error::ErrorForbidden("Access denied"));
    }

    let db_conn_arc = app_state.db.clone();
    let user_id_str = user_id.into_inner();

    let user = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;
        crate::db::get_user_by_id(&conn, &user_id_str)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while fetching user: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to fetch user")
    })?
    .map_err(anyhow_to_actix_error)?;

    match user {
        Some(user) => Ok(HttpResponse::Ok().json(user)),
        None => Ok(HttpResponse::NotFound().json(json!({
            "message": "User not found"
        }))),
    }
}

// PUT /users/{user_id} (admin or self)
pub async fn update_user(
    app_state: web::Data<AppState>,
    auth: Auth,
    user_id: web::Path<String>,
    update: web::Json<models::UserUpdate>,
) -> ActixResult<impl Responder> {
    // Allow if admin or if user is updating their own data
    if auth.role != "admin" && auth.user_id != user_id.as_str() {
        return Err(actix_web::error::ErrorForbidden("Access denied"));
    }

    let db_conn_arc = app_state.db.clone();
    let user_id_str = user_id.into_inner();
    let update_data = update.into_inner();

    web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;
        crate::db::update_user(&conn, &user_id_str, &update_data)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while updating user: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to update user")
    })?
    .map_err(anyhow_to_actix_error)?;

    Ok(HttpResponse::Ok().json(json!({
        "message": "User updated successfully"
    })))
}

// DELETE /users/{user_id} (admin only)
pub async fn delete_user(
    app_state: web::Data<AppState>,
    auth: Auth,
    user_id: web::Path<String>,
) -> ActixResult<impl Responder> {
    // Only admins can delete users
    crate::auth::require_admin(&auth)?;

    let db_conn_arc = app_state.db.clone();
    let user_id_str = user_id.into_inner();

    let deleted = web::block(move || {
        let conn = db_conn_arc
            .lock()
            .map_err(|_| anyhow::anyhow!("Database mutex poisoned"))?;
        crate::db::delete_user(&conn, &user_id_str)
    })
    .await
    .map_err(|e| {
        log::error!("web::block error while deleting user: {:?}", e);
        actix_web::error::ErrorInternalServerError("Failed to delete user")
    })?
    .map_err(anyhow_to_actix_error)?;

    if deleted {
        Ok(HttpResponse::Ok().json(json!({
            "message": "User deleted successfully"
        })))
    } else {
        Ok(HttpResponse::NotFound().json(json!({
            "message": "User not found"
        })))
    }
}
