// src/main.rs
use actix_cors::Cors;
use actix_files as fs;
use actix_route_rate_limiter::{Limiter, RateLimiter};
use actix_web::{http::header, middleware::Logger, web, App, HttpServer};
use config::{Config, Environment};
use dotenv::dotenv;
use std::env;
use std::io::Result as IoResult;
use std::process;
use std::sync::{Arc, Mutex};
use std::time::{Duration, Instant};
use tracing::{error, info, warn};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt};

// Added for throttling map
use std::collections::HashMap;

// Import modules
mod auth;
mod db;
mod handlers;
mod models;
mod notifications;

use notifications::{NotificationConfig, NotificationService};

// --- CAPTCHA Configuration ---
#[derive(Clone, Debug)]
pub struct CaptchaConfig {
    pub enabled: bool,
    pub secret_key: String,
    pub verification_url: String, // e.g., "https://hcaptcha.com/siteverify"
}

impl CaptchaConfig {
    // Function to load from environment variables
    pub fn from_env() -> Result<Self, std::env::VarError> {
        // Return VarError for simplicity
        let enabled = std::env::var("CAPTCHA_ENABLED")
            .map(|v| v.parse().unwrap_or(false))
            .unwrap_or(false); // Default to false if not set or parse error

        // Use Ok variant of Result for keys, default to empty if not found
        let secret_key = std::env::var("CAPTCHA_SECRET_KEY").unwrap_or_default();
        let verification_url = std::env::var("CAPTCHA_VERIFICATION_URL").unwrap_or_default();

        // Basic validation: if enabled, secret key and URL must be present
        if enabled && (secret_key.is_empty() || verification_url.is_empty()) {
            warn!("CAPTCHA_ENABLED is true, but CAPTCHA_SECRET_KEY or CAPTCHA_VERIFICATION_URL is missing. CAPTCHA will be effectively disabled.");
            Ok(Self {
                enabled: false, // Force disable if config is incomplete
                secret_key,
                verification_url,
            })
        } else {
            Ok(Self {
                enabled,
                secret_key,
                verification_url,
            })
        }
    }
}
// --- End CAPTCHA Configuration ---

// Application state that will be shared across all routes
pub struct AppState {
    db: Arc<Mutex<rusqlite::Connection>>,
    notification_service: Arc<NotificationService>,
    captcha_config: CaptchaConfig,
    // Map form_id to the Instant of the last notification attempt for that form
    last_notification_times: Arc<Mutex<HashMap<String, Instant>>>,
    // Map form_id -> ip_address -> (last_attempt_time, count) for rate limiting
    form_submission_attempts: Arc<Mutex<HashMap<String, HashMap<String, (Instant, u32)>>>>,
}

#[actix_web::main]
async fn main() -> IoResult<()> {
    // Load environment variables from .env file
    dotenv().ok();

    // Initialize Sentry for error tracking
    let _guard = sentry::init((
        env::var("SENTRY_DSN").unwrap_or_default(),
        sentry::ClientOptions {
            release: sentry::release_name!(),
            ..Default::default()
        },
    ));

    // Initialize structured logging
    tracing_subscriber::registry()
        .with(tracing_subscriber::EnvFilter::new(
            env::var("RUST_LOG").unwrap_or_else(|_| "info".into()),
        ))
        .with(tracing_subscriber::fmt::layer())
        .init();

    // Load configuration
    let settings = Config::builder()
        .add_source(Environment::default())
        .build()
        .unwrap_or_else(|e| {
            error!("Failed to load configuration: {}", e);
            process::exit(1);
        });

    // --- Configuration (Environment Variables) ---
    let database_url = settings.get_string("DATABASE_URL").unwrap_or_else(|_| {
        warn!("DATABASE_URL environment variable not set. Defaulting to 'form_data.db'.");
        "form_data.db".to_string()
    });

    let bind_address = settings.get_string("BIND_ADDRESS").unwrap_or_else(|_| {
        warn!("BIND_ADDRESS environment variable not set. Defaulting to '127.0.0.1:8080'.");
        "127.0.0.1:8080".to_string()
    });

    // Read allowed origins as a comma-separated string, defaulting to empty
    let allowed_origins_str = env::var("ALLOWED_ORIGIN").unwrap_or_else(|_| {
        warn!("ALLOWED_ORIGIN environment variable not set. CORS will be restrictive.");
        String::new() // Default to empty string if not set
    });

    // Split the string into a vector of origins
    let allowed_origins_list: Vec<String> = if allowed_origins_str.is_empty() {
        Vec::new() // Return an empty vector if the string is empty
    } else {
        allowed_origins_str
            .split(',')
            .map(|s| s.trim().to_string()) // Trim whitespace and convert to String
            .filter(|s| !s.is_empty()) // Remove empty strings resulting from extra commas
            .collect()
    };

    info!(" --- Formies Backend Configuration ---");
    info!("Required Environment Variables:");
    info!(" - DATABASE_URL (Current: {})", database_url);
    info!(" - BIND_ADDRESS (Current: {})", bind_address);
    info!(" - INITIAL_ADMIN_USERNAME (Set during startup, MUST be present)");
    info!(" - INITIAL_ADMIN_PASSWORD (Set during startup, MUST be present)");
    info!("Optional Environment Variables:");
    if !allowed_origins_list.is_empty() {
        info!(
            " - ALLOWED_ORIGIN (Set: {})",
            allowed_origins_list.join(", ") // Log the list nicely
        );
    } else {
        warn!(" - ALLOWED_ORIGIN (Not Set): CORS will be restrictive");
    }
    info!(" - RUST_LOG (e.g., 'info,formies_be=debug')");
    info!(" --- End Configuration ---");

    // Initialize database connection
    let db_connection = match db::init_db(&database_url) {
        Ok(conn) => conn,
        Err(e) => {
            if e.to_string().contains("INITIAL_ADMIN_USERNAME")
                || e.to_string().contains("INITIAL_ADMIN_PASSWORD")
            {
                error!("FATAL: {}", e);
                error!("Please set the INITIAL_ADMIN_USERNAME and INITIAL_ADMIN_PASSWORD environment variables.");
            } else {
                error!(
                    "FATAL: Failed to initialize database at {}: {:?}",
                    database_url, e
                );
            }
            process::exit(1);
        }
    };

    // Initialize rate limiter using the correct fields
    let limiter = Limiter {
        ip_addresses: std::collections::HashMap::new(), // Stores IP request counts
        duration: chrono::TimeDelta::from_std(Duration::from_secs(60)).expect("Invalid duration"), // Convert std::time::Duration
        num_requests: 100, // Max requests allowed in the duration
    };
    // Create the cloneable Arc<Mutex<Limiter>> outside the closure
    let limiter_data = Arc::new(Mutex::new(limiter));

    // Initialize notification service
    let notification_config = NotificationConfig::from_env().unwrap_or_else(|e| {
        warn!(
            "Failed to load notification configuration: {}. Notifications will not be available.",
            e
        );
        NotificationConfig::default()
    });
    let notification_service = Arc::new(NotificationService::new(notification_config));

    // Load CAPTCHA Configuration
    let captcha_config = CaptchaConfig::from_env().unwrap_or_else(|e| {
        warn!(
            "Failed to load CAPTCHA configuration: {}. CAPTCHA will be disabled.",
            e
        );
        // Ensure default is truly disabled
        CaptchaConfig {
            enabled: false,
            secret_key: String::new(),
            verification_url: String::new(),
        }
    });
    if captcha_config.enabled {
        info!("CAPTCHA verification is ENABLED.");
    } else {
        info!("CAPTCHA verification is DISABLED (or required env vars missing).");
    }

    // Create AppState with all services
    let app_state = web::Data::new(AppState {
        db: Arc::new(Mutex::new(db_connection)),
        notification_service: notification_service.clone(),
        captcha_config: captcha_config.clone(),
        last_notification_times: Arc::new(Mutex::new(HashMap::new())),
        form_submission_attempts: Arc::new(Mutex::new(HashMap::new())), // Initialize rate limit map
    });

    info!("Starting server at http://{}", bind_address);

    HttpServer::new(move || {
        let app_state = app_state.clone(); // This now includes captcha_config
        let allowed_origins = allowed_origins_list.clone();
        let rate_limiter = RateLimiter::new(limiter_data.clone());

        // Configure CORS
        let cors = if !allowed_origins.is_empty() {
            info!("Configuring CORS for origins: {:?}", allowed_origins);
            let mut cors = Cors::default();
            for origin in allowed_origins {
                cors = cors.allowed_origin(&origin); // Add each origin
            }
            cors.allowed_methods(vec!["GET", "POST", "PUT", "DELETE", "OPTIONS"])
                .allowed_headers(vec![
                    header::AUTHORIZATION,
                    header::ACCEPT,
                    header::CONTENT_TYPE,
                    header::ORIGIN,
                    header::ACCESS_CONTROL_REQUEST_METHOD,
                    header::ACCESS_CONTROL_REQUEST_HEADERS,
                ])
                .supports_credentials()
                .max_age(3600)
        } else {
            warn!("CORS is configured restrictively: No ALLOWED_ORIGIN set.");
            Cors::default() // Keep restrictive default if no origins are provided
                .allowed_methods(vec!["GET", "POST", "PUT", "DELETE", "OPTIONS"])
                .allowed_headers(vec![
                    header::AUTHORIZATION,
                    header::ACCEPT,
                    header::CONTENT_TYPE,
                    header::ORIGIN,
                    header::ACCESS_CONTROL_REQUEST_METHOD,
                    header::ACCESS_CONTROL_REQUEST_HEADERS,
                ])
                .supports_credentials()
                .max_age(3600)
        };

        // Configure JSON payload limits (e.g., 1MB)
        let json_config = web::JsonConfig::default().limit(1024 * 1024);

        App::new()
            .wrap(cors)
            .wrap(Logger::default())
            .wrap(tracing_actix_web::TracingLogger::default())
            .wrap(rate_limiter)
            .app_data(app_state) // Share app state (db, notifications, captcha)
            .app_data(json_config) // Add JSON payload configuration
            .service(
                web::scope("/api")
                    // Health check endpoint
                    .route("/health", web::get().to(handlers::health_check))
                    // Public routes
                    .route("/login", web::post().to(handlers::login))
                    .route("/register", web::post().to(handlers::register))
                    .route(
                        "/forms/{form_id}/submissions",
                        web::post().to(handlers::submit_form),
                    )
                    // Protected routes
                    .route("/logout", web::post().to(handlers::logout))
                    .route("/forms", web::post().to(handlers::create_form))
                    .route("/forms", web::get().to(handlers::get_forms))
                    .route(
                        "/forms/{form_id}/submissions",
                        web::get().to(handlers::get_submissions),
                    )
                    .route(
                        "/forms/{form_id}/notifications",
                        web::get().to(handlers::get_notification_settings),
                    )
                    .route(
                        "/forms/{form_id}/notifications",
                        web::put().to(handlers::update_notification_settings),
                    )
                    // User management routes
                    .route("/users", web::get().to(handlers::list_users))
                    .route("/users/{user_id}", web::get().to(handlers::get_user))
                    .route("/users/{user_id}", web::put().to(handlers::update_user))
                    .route("/users/{user_id}", web::delete().to(handlers::delete_user)),
            )
            .service(
                fs::Files::new("/", "./frontend/")
                    .index_file("index.html")
                    .use_last_modified(true)
                    .default_handler(fs::NamedFile::open("./frontend/index.html").unwrap_or_else(
                        |_| {
                            error!("Fallback file not found: ../frontend/index.html");
                            process::exit(1);
                        },
                    )),
            )
    })
    .bind(&bind_address)?
    .run()
    .await
}
