// src/models.rs
use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};
// Consider adding chrono for DateTime types if needed in responses
// use chrono::{DateTime, Utc};

// Represents the structure for defining a form
#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Form {
    #[serde(skip_serializing_if = "Option::is_none")]
    pub id: Option<String>,
    pub name: String,
    /// Stores the structure defining the form fields.
    /// Expected to be a JSON array of field definition objects.
    /// Example field definition object:
    /// ```json
    /// {
    ///   "name": "email",                  // String, required: Unique identifier for the field
    ///   "type": "email",                  // String, required: "string", "number", "boolean", "email", "url", "object", "array"
    ///   "label": "Email Address",         // String, optional: User-friendly label
    ///   "required": true,                 // Boolean, optional (default: false): If the field must have a value
    ///   "placeholder": "you@example.com", // String, optional: Placeholder text
    ///   "minLength": 5,                   // Number, optional: Minimum length for strings
    ///   "maxLength": 100,                 // Number, optional: Maximum length for strings
    ///   "min": 0,                         // Number, optional: Minimum value for numbers
    ///   "max": 100,                       // Number, optional: Maximum value for numbers
    ///   "pattern": "^\\S+@\\S+\\.\\S+$"     // String, optional: Regex pattern for strings (e.g., email, url types might use this implicitly or explicitly)
    ///   // Add other properties like "options" for select/radio, etc.
    /// }
    /// ```
    pub fields: serde_json::Value,
    pub owner_id: String,
    pub notify_email: Option<String>,
    pub notify_ntfy_topic: Option<String>,
    pub created_at: DateTime<Utc>,
}

// Represents a single submission for a specific form
#[derive(Serialize, Deserialize, Debug, Clone)]
pub struct Submission {
    pub id: String,
    pub form_id: String,
    /// Stores the data submitted by the user.
    /// Expected to be a JSON object where keys are field names (`name`) from the form definition's `fields` array.
    /// Example: `{ "email": "user@example.com", "age": 30 }`
    pub data: serde_json::Value,
    pub created_at: DateTime<Utc>,
}

// Used for the /login endpoint request body
#[derive(Debug, Serialize, Deserialize)]
pub struct LoginCredentials {
    pub username: String,
    pub password: String,
}

// Used for the /login endpoint response body
#[derive(Debug, Serialize, Deserialize)]
pub struct LoginResponse {
    pub token: String, // The session token (UUID)
}

// Used internally to represent a user fetched from the DB for authentication check
// Not serialized, only used within db.rs and handlers.rs
#[derive(Debug)]
pub struct UserAuthData {
    pub id: String,
    pub hashed_password: String,
    // Note: Token and expiry are handled separately and not needed in this specific struct
}

// Used for the GET/PUT /forms/{form_id}/notifications endpoints
#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct NotificationSettingsPayload {
    pub notify_email: Option<String>,
    pub notify_ntfy_topic: Option<String>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct User {
    pub id: String,
    pub username: String,
    #[serde(skip_serializing)] // Never send password in responses
    pub password: Option<String>,
    pub role: String,
    pub created_at: DateTime<Utc>,
}

// Used for user registration
#[derive(Debug, Serialize, Deserialize)]
pub struct UserRegistration {
    pub username: String,
    pub password: String,
}

// Used for user profile updates
#[derive(Debug, Serialize, Deserialize)]
pub struct UserUpdate {
    pub username: Option<String>,
    pub password: Option<String>,
}
