use anyhow::Result;
use lettre::message::header::ContentType;
use lettre::transport::smtp::authentication::Credentials;
use lettre::{Message, SmtpTransport, Transport};
use serde::Serialize;
use std::env;

#[derive(Debug, Serialize)]
pub struct NotificationConfig {
    smtp_host: String,
    smtp_port: u16,
    smtp_username: String,
    smtp_password: String,
    from_email: String,
    ntfy_topic: String,
    ntfy_server: String,
}

impl Default for NotificationConfig {
    fn default() -> Self {
        Self {
            smtp_host: String::new(),
            smtp_port: 587,
            smtp_username: String::new(),
            smtp_password: String::new(),
            from_email: String::new(),
            ntfy_topic: String::new(),
            ntfy_server: "https://ntfy.sh".to_string(),
        }
    }
}

impl NotificationConfig {
    pub fn from_env() -> Result<Self> {
        Ok(Self {
            smtp_host: env::var("SMTP_HOST")?,
            smtp_port: env::var("SMTP_PORT")?.parse()?,
            smtp_username: env::var("SMTP_USERNAME")?,
            smtp_password: env::var("SMTP_PASSWORD")?,
            from_email: env::var("FROM_EMAIL")?,
            ntfy_topic: env::var("NTFY_TOPIC")?,
            ntfy_server: env::var("NTFY_SERVER").unwrap_or_else(|_| "https://ntfy.sh".to_string()),
        })
    }

    pub fn is_email_configured(&self) -> bool {
        !self.smtp_host.is_empty()
            && !self.smtp_username.is_empty()
            && !self.smtp_password.is_empty()
            && !self.from_email.is_empty()
    }

    pub fn is_ntfy_configured(&self) -> bool {
        !self.ntfy_topic.is_empty()
    }
}

pub struct NotificationService {
    config: NotificationConfig,
}

impl NotificationService {
    pub fn new(config: NotificationConfig) -> Self {
        Self { config }
    }

    pub async fn send_email(&self, to: &str, subject: &str, body: &str) -> Result<()> {
        if !self.config.is_email_configured() {
            return Ok(());
        }

        let email = Message::builder()
            .from(self.config.from_email.parse()?)
            .to(to.parse()?)
            .subject(subject)
            .header(ContentType::TEXT_PLAIN)
            .body(body.to_string())?;

        let creds = Credentials::new(
            self.config.smtp_username.clone(),
            self.config.smtp_password.clone(),
        );

        let mailer = SmtpTransport::relay(&self.config.smtp_host)?
            .port(self.config.smtp_port)
            .credentials(creds)
            .build();

        mailer.send(&email)?;
        Ok(())
    }

    pub fn send_ntfy(&self, title: &str, message: &str, priority: Option<u8>) -> Result<()> {
        if !self.config.is_ntfy_configured() {
            return Ok(());
        }

        let url = format!("{}/{}", self.config.ntfy_server, self.config.ntfy_topic);

        let mut request = ureq::post(&url).set("Title", title);

        if let Some(p) = priority {
            request = request.set("Priority", &p.to_string());
        }

        request.send_string(message)?;
        Ok(())
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_notification_config() {
        std::env::set_var("SMTP_HOST", "smtp.example.com");
        std::env::set_var("SMTP_PORT", "587");
        std::env::set_var("SMTP_USERNAME", "test@example.com");
        std::env::set_var("SMTP_PASSWORD", "password");
        std::env::set_var("FROM_EMAIL", "noreply@example.com");
        std::env::set_var("NTFY_TOPIC", "my-topic");

        let config = NotificationConfig::from_env().unwrap();
        assert_eq!(config.smtp_host, "smtp.example.com");
        assert_eq!(config.smtp_port, 587);
        assert_eq!(config.ntfy_server, "https://ntfy.sh");
    }

    #[test]
    fn test_config_validation() {
        let default_config = NotificationConfig::default();
        assert!(!default_config.is_email_configured());
        assert!(!default_config.is_ntfy_configured());

        let config = NotificationConfig {
            smtp_host: "smtp.example.com".to_string(),
            smtp_port: 587,
            smtp_username: "user".to_string(),
            smtp_password: "pass".to_string(),
            from_email: "test@example.com".to_string(),
            ntfy_topic: "topic".to_string(),
            ntfy_server: "https://ntfy.sh".to_string(),
        };
        assert!(config.is_email_configured());
        assert!(config.is_ntfy_configured());
    }
}
